.-vala-panel-appmenu-private {
    background-color: transparent;
    background-image: none;
    color: inherit;
    box-shadow: inset 0 0px;
}

.-vala-panel-appmenu-core scrolledwindow,
.-vala-panel-appmenu-private > menuitem,
.-vala-panel-appmenu-private > menuitem:first-child > label {
    margin-top: 0;
    margin-bottom: 0;
    padding-top: 0;
    padding-bottom: 0;
}

menubar.-vala-panel-appmenu-bold > menuitem:first-child > label {
   font-weight: bolder;
}

.-vala-panel-appmenu-private > separator {
	
    background-color: transparent;
    background-image: none;
	opacity: 0;
}
